@@ -60,7 +60,10 @@
             sweep_top_level_server/2,
             sweep_top_level_threads/2,
             sweep_accept_top_level_client/2,
-            write_sweep_module_location/0
+            sweep_local_predicate_export_comment/2,
+            write_sweep_module_location/0,
+            sweep_module_html_documentation/2,
+            sweep_predicate_html_documentation/2
           ]).
 
 :- use_module(library(pldoc)).
@@ -71,11 +74,11 @@
 :- use_module(library(pldoc/doc_wiki)).
 :- use_module(library(pldoc/doc_modes)).
 :- use_module(library(pldoc/doc_man)).
+:- use_module(library(pldoc/doc_html)).
 :- use_module(library(pldoc/man_index)).
 :- use_module(library(lynx/html_text)).
+:- use_module(library(http/html_write)).
 :- use_module(library(prolog_pack)).
-:- use_module(library(help)).
-:- use_module(library(prolog_server)).
 
 :- meta_predicate with_buffer_stream(-, +, 0).
 
@@ -237,6 +240,7 @@ sweep_identifier_at_point_(Path0, Point, Contents, Identifier) :-
     prolog_colourise_term(Contents, Path,
                           sweep_handle_identifier_at_point(Path, M, Point),
                           []),
+    !,
     sweep_current_identifier_at_point(Identifier0),
     term_string(Identifier0, Identifier).
 
@@ -316,7 +320,7 @@ sweep_handle_identifier_at_point_goal(_Path, _M0, undefined, M:Goal) :-
 sweep_handle_identifier_at_point_goal(_Path, _M0, undefined, Goal) :-
     !,
     pi_head(PI, Goal),
-    asserta(sweep_current_identifier_at_point(undefined:PI)).
+    asserta(sweep_current_identifier_at_point(PI)).
 sweep_handle_identifier_at_point_goal(_Path, _M0, meta, _:Goal) :-
     !,
     pi_head(PI, Goal),
@@ -426,9 +430,45 @@ sweep_module_path(ModuleName, Path) :-
 
 sweep_module_path_(Module, Path) :-
     module_property(Module, file(Path)), !.
+sweep_module_path_(Module, Path) :-
+    xref_module(Path, Module), !.
 sweep_module_path_(Module, Path) :-
     '$autoload':library_index(_, Module, Path0), !, string_concat(Path0, ".pl", Path).
 
+sweep_predicate_html_documentation(P0, D) :-
+    term_string(P1, P0),
+    (   P1 = M:F/N
+    ->  true
+    ;   P1 = F/N, M = system
+    ),
+    (   (   current_module(M)
+        ;   xref_module(_, M)
+        )
+    ->  true
+    ;   '$autoload':library_index(_, M, Path),
+        xref_source(Path, [comments(store)])
+    ),
+    (   M == system
+    ->  pldoc_man:load_man_object(F/N, _, _, DOM)
+    ;   doc_comment(M:F/N, Pos, _, Comment),
+        pldoc_html:pred_dom(M:F/N, [], Pos-Comment, DOM)
+    ),
+    phrase(pldoc_html:html(DOM), HTML),
+    with_output_to(string(D), html_write:print_html(HTML)).
+
+sweep_module_html_documentation(M0, D) :-
+    atom_string(M, M0),
+    (   (   current_module(M)
+        ;   xref_module(_, M)
+        )
+    ->  true
+    ;   '$autoload':library_index(_, M, Path),
+        xref_source(Path, [comments(store)])
+    ),
+    doc_comment(M:module(Desc), Pos, _, Comment),
+    pldoc_html:pred_dom(M:module(Desc), [], Pos-Comment, DOM),
+    phrase(pldoc_html:html(DOM), HTML),
+    with_output_to(string(D), html_write:print_html(HTML)).
 
 sweep_modules_collection([], Modules) :-
     findall([M|P], ( module_property(M, file(P0)), atom_string(P0, P) ), Modules0, Tail),
@@ -610,7 +650,7 @@ sweep_predicate_matches(Sub, [String|_]) :-
     sub_string(String, _, _, _, Sub).
 
 sweep_predicate_non_hidden([String|_]) :-
-    \+ sub_string(String, _, _, _, ":'$").
+    \+ sub_string(String, _, _, _, "$").
 
 sweep_predicate_description(M:F/N, [S|T]) :-
     sweep_predicate_description_(M, F, N, T),
@@ -780,11 +820,21 @@ sweep_imenu_index(Path, Index) :-
             ),
             Index).
 
+:- if(exists_source(library(sweep_link))).
+:- use_module(library(sweep_link), [write_sweep_module_location/0]).
+:- else.
 write_sweep_module_location :-
+    format('V ~w~n', 1),
     absolute_file_name(foreign('sweep-module'),
                        Path,
                        [file_type(executable), access(read)]),
-    writeln(Path).
+    (   current_prolog_flag(executable_format, elf)
+    ->  current_prolog_flag(libswipl, Libpath),
+        format('L ~w~n', Libpath)
+    ;   true
+    ),
+    format('M ~w~n', Path).
+:- endif.
 
 sweep_top_level_server(_, Port) :-
     tcp_socket(ServerSocket),
@@ -854,3 +904,23 @@ sweep_accept_top_level_client(Buffer, _) :-
 sweep_thread_signal([ThreadId|Goal0], _) :-
     term_string(Goal, Goal0),
     thread_signal(ThreadId, Goal).
+
+sweep_local_predicate_export_comment([Path0,F0,A],Comm) :-
+    atom_string(Path, Path0),
+    atom_string(F, F0),
+    doc_comment(_:F/A, Path:_, _Summary, Comment),
+    comment_modes(Comment, Modes),
+    pi_head(F/A, Head),
+    member(ModeAndDet, Modes),
+    strip_det(ModeAndDet, Head),
+    Head =.. [_|Args],
+    Head2 =.. ['DUMMY'|Args],
+    term_string(Head2, Syn0,
+                [ module(pldoc_modes),
+                  quoted(false)
+                ]),
+    sub_string(Syn0, 6, _, 1, Comm).
+
+strip_det(Mode is _, Mode) :- !.
+strip_det(//(Mode), Mode) :- !.
+strip_det(Mode, Mode).
