@@ -3,8 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2005-2018-2022, University of Amsterdam
-                                   SWI-Prolog Solutions b.v.
+    Copyright (c)  2018-2022, University of Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@
 :- module(test_xpce,
           [ test_xpce/0
           ]).
+:- encoding(utf8).
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XPCE/SWI-Prolog test file.  A test is a clause of the form:
@@ -210,8 +211,7 @@ srcsink(read-1) :-
                  *             FILE             *
                  *******************************/
 
-foreign(Name) :-
-    atom_codes(Name, [1087, 1083, 1072, 1090, 1085, 1072, 1103]).
+foreign(привіт).
 
 file(env-1) :-
     new(F, file('$PCEHOME/Defaults')),
@@ -245,8 +245,14 @@ file(exists-1) :-
                  *         UNICODE FILE         *
                  *******************************/
 
+foreign_creatable(Name) :-
+    current_prolog_flag(tmp_dir, TmpDir),
+    current_prolog_flag(pid, PID),
+    foreign(Base),
+    format(atom(Name), '~w/swipl-~w-~d', [TmpDir,Base,PID]).
+
 unicode_file(utf8-1) :-
-    foreign(Name),
+    foreign_creatable(Name),
     Text = 'Hello world\n',
     new(F, file(Name)),
     send(F, open, write),
@@ -306,22 +312,24 @@ dir(members-3) :-
                  *******************************/
 
 unicode_dir(foreign-1) :-
-    foreign(Name),
+    foreign_creatable(Name),
     new(D, directory(Name)),
     send(D, make),
     send(D, exists),
     delete_directory(Name).
 unicode_dir(foreign-2) :-
-    foreign(Name),
+    foreign_creatable(Name),
     new(D, directory(Name)),
     send(D, make),
     send(D, exists),
-    new(D2, directory(.)),
+    file_directory_name(Name, Dir),
+    file_base_name(Name, Base),
+    new(D2, directory(Dir)),
     get_chain(D2, directories, Dirs),
-    member(Name, Dirs),
+    memberchk(Base, Dirs),
     delete_directory(Name).
 unicode_dir(foreign-3) :-
-    foreign(Name),
+    foreign_creatable(Name),
     new(D, directory(Name)),
     send(D, make),
     send(D, exists),
@@ -760,7 +768,7 @@ display_present :-
 test_xpce :-
     retractall(failed(_)),
     retractall(blocked(_,_)),
-    forall(testset(Set),            % force XOCE incremental GC
+    forall(testset(Set),            % force XPCE incremental GC
            send(@prolog, call, 'xpce_runtest', Set)),
     scripts,
     report_blocked,
