@@ -154,7 +154,12 @@ query_loop(atom_t goal, int loop)
     }
 
     if ( !rc && (except = PL_exception(qid)) )
-    { restore_after_exception(except);
+    { if ( Sferror(Suser_input) ||
+	   Sferror(Suser_output) ||
+	   Sferror(Suser_error) )
+	return -1;
+
+      restore_after_exception(except);
       rc = -1;
     }
 
